@JS()
library selectize;

import "package:js/js.dart";
import "package:func/func.dart";
import "dart:html" show HtmlElement;
import 'dart:js';
import 'package:js/js_util.dart';

/// Type definitions for Selectize 0.12.13
/// Project: https://github.com/brianreavis/selectize.js
/// Definitions by: Adi Dahiya <https://github.com/adidahiya>, Natalie Bausch <https://github.com/naBausch>
/// Definitions: https://github.com/DefinitelyTyped/DefinitelyTyped

/// <reference path="../jquery/jquery.d.ts"/>

// Module Selectize
/// see https://github.com/brianreavis/selectize.js/blob/master/docs/usage.md
/// option identifiers are parameterized by T; data is parameterized by U
@anonymous
@JS()
class IOptions<T, U> {
  /// General
  /// ------------------------------------------------------------------------------------------------------------
  /// The string to separate items by. This option is only used when Selectize is instantiated from a
  /// <input type="text"> element.
  /// Default: ','
  external String get delimiter;
  external set delimiter(String v);

  /// Enable or disable international character support.
  /// Default: true
  external bool get diacritics;
  external set diacritics(bool v);

  /// Allows the user to create a new items that aren't in the list of options.
  /// This option can be any of the following: "true", "false" (disabled), or a function that accepts two
  /// arguments: "input" and "callback". The callback should be invoked with the final data for the option.
  /// Default: false
  external dynamic get create;
  external set create(dynamic v);

  /// If true, when user exits the field (clicks outside of input or presses ESC) new option is created and
  /// selected (if `create`-option is enabled).
  /// Default: false
  external bool get createOnBlur;
  external set createOnBlur(bool v);

  /// Specifies a RegExp or String containing a regular expression that the current search filter must match to
  /// be allowed to be created. May also be a predicate function that takes the filter text and returns whether
  /// it is allowed.
  /// Default: null
  external dynamic get createFilter;
  external set createFilter(dynamic v);

  /// Toggles match highlighting within the dropdown menu.
  /// Default: true
  external bool get highlight;
  external set highlight(bool v);

  /// If false, items created by the user will not show up as available options once they are unselected.
  /// Default: true
  external bool get persist;
  external set persist(bool v);

  /// Show the dropdown immediately when the control receives focus.
  /// Default: true
  external bool get openOnFocus;
  external set openOnFocus(bool v);

  /// The max number of items to render at once in the dropdown list of options.
  /// Default: 1000
  external num get maxOptions;
  external set maxOptions(num v);

  /// The max number of items the user can select.
  /// Default: Infinity
  external num get maxItems;
  external set maxItems(num v);

  /// If true, the items that are currently selected will not be shown in the dropdown list of available options.
  /// Default: false
  external bool get hideSelected;
  external set hideSelected(bool v);

  /// If true, Selectize will treat any options with a "" value like normal. This defaults to false to
  /// accomodate the common <select> practice of having the first empty option act as a placeholder.
  /// Default: false
  external bool get allowEmptyOption;
  external set allowEmptyOption(bool v);

  /// The animation duration (in milliseconds) of the scroll animation triggered when going [up] and [down] in
  /// the options dropdown.
  /// Default: 60
  external num get scrollDuration;
  external set scrollDuration(num v);

  /// The number of milliseconds to wait before requesting options from the server or null.
  /// If null, throttling is disabled.
  /// Default: 300
  external num get loadThrottle;
  external set loadThrottle(num v);

  /// If true, the "load" function will be called upon control initialization (with an empty search).
  /// Alternatively it can be set to "focus" to call the "load" function when control receives focus.
  /// Default: false
  external dynamic get preload;
  external set preload(dynamic v);

  /// The element the dropdown menu is appended to. This should be "body" or null.
  /// If null, the dropdown will be appended as a child of the selectize control.
  /// Default: null
  external String get dropdownParent;
  external set dropdownParent(String v);

  /// Sets if the "Add..." option should be the default selection in the dropdown.
  /// Default: false
  external bool get addPrecedence;
  external set addPrecedence(bool v);

  /// If true, the tab key will choose the currently selected item.
  /// Default: false
  external bool get selectOnTab;
  external set selectOnTab(bool v);

  /// An array of plugins to use
  /// Default: null
  external List<String> get plugins;
  external set plugins(List<String> v);

  /// Data / Searching
  /// ------------------------------------------------------------------------------------------------------------
  /// Options available to select; array of objects. If your element is <select> with <option>s specified this
  /// property gets populated accordingly. Setting this property is convenient if you have your data as an
  /// array and want to automatically generate the <option>s.
  /// Default: []
  external List<U> get options;
  external set options(List<U> v);

  /// The <option> attribute from which to read JSON data about the option.
  /// Default: "data-data"
  external String get dataAttr;
  external set dataAttr(String v);

  /// The name of the property to use as the "value" when an item is selected.
  /// Default: "value"
  external String get valueField;
  external set valueField(String v);

  /// Option groups that options will be bucketed into.
  /// If your element is a <select> with <optgroup>s this property gets populated automatically.
  /// Make sure each object in the array has a property named whatever "optgroupValueField" is set to.
  external List<U> get optgroups;
  external set optgroups(List<U> v);

  /// The name of the option group property that serves as its unique identifier.
  /// Default: "value"
  external String get optgroupValueField;
  external set optgroupValueField(String v);

  /// The name of the property to render as an option / item label (not needed when custom rendering
  /// functions are defined).
  /// Default: "text"
  external String get labelField;
  external set labelField(String v);

  /// The name of the property to render as an option group label (not needed when custom rendering
  /// functions are defined).
  /// Default: "label"
  external String get optgroupLabelField;
  external set optgroupLabelField(String v);

  /// The name of the property to group items by.
  /// Default: "optgroup"
  external String get optgroupField;
  external set optgroupField(String v);

  /// A single field or an array of fields to sort by. Each item in the array should be an object containing at
  /// least a "field" property. Optionally, "direction" can be set to "asc" or "desc". The order of the array
  /// defines the sort precedence.
  /// Unless present, a special "$score" field will be automatically added to the beginning of the sort list.
  /// This will make results sorted primarily by match quality (descending).
  /// Default: "$order"
  external dynamic get sortField;
  external set sortField(dynamic v);

  /// An array of property names to analyze when filtering options.
  /// Default: ["text"]
  external dynamic get searchField;
  external set searchField(dynamic v);

  /// When searching for multiple terms (separated by a space), this is the operator used. Can be "and" or "or".
  /// Default: "and"
  external String get searchConjunction;
  external set searchConjunction(String v);

  /// An array of optgroup values that indicates the order they should be listed in in the dropdown.
  /// If not provided, groups will be ordered by the ranking of the options within them.
  /// Default: null
  external List<String> get optgroupOrder;
  external set optgroupOrder(List<String> v);

  /// Copy the original input classes to the Dropdown element.
  /// Default: true
  external bool get copyClassesToDropdown;
  external set copyClassesToDropdown(bool v);

  /// Callbacks
  /// ------------------------------------------------------------------------------------------------------------
  /// Invoked when new options should be loaded from the server.
  external dynamic load(String query, Function callback);

  /// Overrides the scoring function used to sort available options. The provided function should return a
  /// function that returns a number greater than or equal to zero to represent the "score" of an item
  /// (the function's first argument). If 0, the option is declared not a match.
  external Func1<dynamic, num> score(ISearch search);

  /// Invoked once the control is completely initialized.
  external dynamic onInitialize();

  /// Invoked when the value of the control changes.
  /// If single select, value is of type T.
  /// If multi select, value is of type T[].
  external dynamic onChange(dynamic value);

  /// Invoked when an item is selected.
  external dynamic onItemAdd(T value, JQuery item);

  /// Invoked when an item is deselected.
  external dynamic onItemRemove(T value);

  /// Invoked when the control is manually cleared via the clear() method.
  external dynamic onClear();

  /// Invoked when the user attempts to delete the current selection.
  external dynamic onDelete(List<T> values);

  /// Invoked when a new option is added to the available options list.
  external dynamic onOptionAdd(T value, U data);

  /// Invoked when an option is removed from the available options.
  external dynamic onOptionRemove(T value);

  /// Invoked when the dropdown opens.
  external dynamic onDropdownOpen(JQuery dropdown);

  /// Invoked when the dropdown closes.
  external dynamic onDropdownClose(JQuery dropdown);

  /// Invoked when the user types while filtering options.
  external dynamic onType(String srt);

  /// Invoked when new options have been loaded and added to the control (via the "load" option or "load" API method).
  external dynamic onLoad(List<U> data);

  /// Rendering
  /// ------------------------------------------------------------------------------------------------------------
  external ICustomRenderers<U> get render;
  external set render(ICustomRenderers<U> v);
}

/// Custom rendering functions. Each function should accept two arguments: "data" and "escape" and return
/// HTML (string) with a single root element. The "escape" argument is a function that takes a string and
/// escapes all special HTML characters. This is very important to use to prevent XSS vulnerabilities.
@anonymous
@JS()
abstract class ICustomRenderers<U> {
  /// An option in the dropdown list of available options.
  external String option(U data, String escape(String input));

  /// An item the user has selected.
  external String item(U data, String escape(String input));

  /// The "create new" option at the bottom of the dropdown. The data contains one property: "input"
  /// (which is what the user has typed).
  external String option_create(U data, String escape(String input));

  /// The header of an option group.
  external String optgroup_header(U data, String escape(String input));

  /// The wrapper for an optgroup. The "html" property in the data will be the raw html of the optgroup's header
  /// and options.
  external String optgroup(U data, String escape(String input));
}

class CustomRenderers extends ICustomRenderers<String> {}

/// see https://github.com/brianreavis/selectize.js/blob/master/docs/api.md
@anonymous
@JS()
abstract class IApi<T, U> {
  /// Dropdown Options
  /// ------------------------------------------------------------------------------------------------------------
  /// Adds an available option. If it already exists, nothing will happen.
  /// Note: this does not refresh the options list dropdown (use refreshOptions() for that).
  external void addOption(U data);

  /// Updates an option available for selection. If it is visible in the selected items or options dropdown,
  /// it will be re-rendered automatically.
  external void updateOption(T value, U data);

  /// Removes the option identified by the given value.
  external void removeOption(T value);

  /// Removes all options from the control.
  external void clearOptions();

  /// Retrieves the jQuery element for the option identified by the given value.
  external dynamic getOption(T value);

  /// Retrieves the jQuery element for the previous or next option, relative to the currently highlighted option.
  /// The "direction" argument should be 1 for "next" or -1 for "previous".
  external void getAdjacentOption(T value, num direction);

  /// Refreshes the list of available options shown in the autocomplete dropdown menu.
  external void refreshOptions(bool triggerDropdown);

  /// Selected Items
  /// ------------------------------------------------------------------------------------------------------------
  /// Resets / clears all selected items from the control.
  external void clear();

  /// Returns the jQuery element of the item matching the given value.
  external JQuery getItem(T value);

  /// "Selects" an item. Adds it to the list at the current caret position.
  external void addItem(T value);

  /// Removes the selected item matching the provided value.
  external void removeItem(T value);

  /// Invokes the "create" method provided in the selectize options that should provide the data for the
  /// new item, given the user input. Once this completes, it will be added to the item list.
  external void createItem(T value);

  /// Re-renders the selected item lists.
  external void refreshItems();

  /// Optgroups
  /// ------------------------------------------------------------------------------------------------------------
  /// Registers a new optgroup for options to be bucketed into.
  /// The "id" argument refers to a value of the property in option identified by the "optgroupField" setting.
  external void addOptionGroup(String id, U data);

  /// Events
  /// ------------------------------------------------------------------------------------------------------------
  /// Adds an event listener.
  external void on(String eventName, Function handler);

  /// Removes an event listener. If no handler is provided, all event listeners are removed.
  external void off(String eventName, [Function handler]);

  /// Triggers event listeners.
  external void trigger(String eventName, [dynamic args1, dynamic args2, dynamic args3, dynamic args4, dynamic args5]);

  /// Dropdown Actions
  /// ------------------------------------------------------------------------------------------------------------
  /// Shows the autocomplete dropdown containing the available options.
  external void open();

  /// Closes the autocomplete dropdown menu.
  external void close();

  /// Calculates and applies the appropriate position of the dropdown.
  external void positionDropdown();

  /// Other
  /// ------------------------------------------------------------------------------------------------------------
  /// Destroys the control and unbinds event listeners so that it can be garbage collected.
  external void destroy();

  /// Loads options by invoking the the provided function. The function should accept one argument (callback)
  /// and invoke the callback with the results once they are available.
  external void load(dynamic callback(dynamic results));

  /// Brings the control into focus.
  external void focus();

  /// Forces the control out of focus.
  external void blur();

  /// Disables user input on the control (note: the control can still receive focus).
  external void lock();

  /// Re-enables user input on the control.
  external void unlock();

  /// Disables user input on the control completely. While disabled, it cannot receive focus.
  external void disable();

  /// Enables the control so that it can respond to focus and user input.
  external void enable();

  /// Returns the value of the control. If multiple items can be selected (e.g. <select multiple>), this
  /// returns an array. If only one item can be selected, this returns a string.
  external dynamic getValue();

  /// Resets the selected items to the given value(s).
  /*external void setValue(T value);*/
  /*external void setValue(List<T> value);*/
  external void setValue(dynamic /*T|List<T>*/ value);

  /// Moves the caret to the specified position ("index" being the index in the list of selected items).
  external void setCaret(num index);

  /// Returns whether or not the user can select more items.
  external bool isFull();

  /// Clears the render cache. Takes an optional template argument (e.g. "option", "item") to clear only that cache.
  external void clearCache([String template]);

  /// When the `settings.placeholder` value is changed, the new placeholder will be displayed.
  external void updatePlaceholder();
}

@anonymous
@JS()
abstract class ISearchToken {
  external RegExp get regex;
  external set regex(RegExp v);
  external String get string;
  external set string(String v);
  external factory ISearchToken({RegExp regex, String string});
}

@anonymous
@JS()
abstract class ISearchResult {
  external String get id;
  external set id(String v);
  external num get score;
  external set score(num v);
  external factory ISearchResult({String id, num score});
}

@anonymous
@JS()
abstract class ISearch {
  /// Original search options.
  external dynamic get options;
  external set options(dynamic v);

  /// The raw user input
  external String get query;
  external set query(String v);

  /// An array containing parsed search tokens. A token is an object containing two properties: "string" and "regex".
  external List<ISearchToken> get tokens;
  external set tokens(List<ISearchToken> v);

  /// The total number of results.
  external num get total;
  external set total(num v);

  /// A list of matched results. Each result is an object containing two properties: "score" and "id".
  external List<ISearchResult> get items;
  external set items(List<ISearchResult> v);
  external factory ISearch(
      {dynamic options, String query, List<ISearchToken> tokens, num total, List<ISearchResult> items});
}

// End module Selectize
@anonymous
@JS()
abstract class JQuery extends IApi {
  external JQuery selectize([IOptions<dynamic, dynamic> options]);
}

@JS("\$")
external JQuery jquery(String selector);

class JsMap {
  dynamic _jsObject;

  JsMap(this._jsObject);

  operator [](String key) => getProperty(_jsObject, key);
//  operator [](int key) => getProperty(_jsObject, key);

  operator []=(String key, value) {
    setProperty(_jsObject, key, value);
  }
}

/* Skipping class HTMLElement*/
